# Copyright (C) 2010-2011 Yaco Sistemas (http://www.yaco.es)
# Copyright (C) 2009 Lorenzo Gil Sanchez <lorenzo.gil.sanchez@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

try:
    from xml.etree import ElementTree
except ImportError:
    from elementtree import ElementTree

from django.conf import settings
from django.contrib import auth
from django.contrib.auth.decorators import login_required
from django.contrib.auth.views import logout as django_logout
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
try:
    from django.views.decorators.csrf import csrf_exempt
except ImportError:
    # Django 1.0 compatibility
    def csrf_exempt(view_func):
        return view_func

from saml2 import BINDING_HTTP_REDIRECT
from saml2.client import Saml2Client
from saml2.metadata import entity_descriptor

from djangosaml2.cache import IdentityCache, OutstandingQueriesCache
from djangosaml2.cache import StateCache
from djangosaml2.conf import config_settings_loader
from djangosaml2.signals import post_authenticated


def _set_subject_id(session, subject_id):
    session['_saml2_subject_id'] = subject_id


def _get_subject_id(session):
    return session['_saml2_subject_id']


def get_custom_setting(name, default=None):
    if hasattr(settings, name):
        return getattr(settings, name)
    else:
        return default


def login(request,
          config_loader=config_settings_loader,
          wayf_template='djangosaml2/wayf.html',
          authorization_error_template='djangosaml2/auth_error.html'):
    """SAML Authorization Request initiator

    This view initiates the SAML2 Authorization handshake
    using the pysaml2 library to create the AuthnRequest.
    It uses the SAML 2.0 Http Redirect protocol binding.
    """
    came_from = request.GET.get('next', settings.LOGIN_REDIRECT_URL)

    if not request.user.is_anonymous():
        return render_to_response(authorization_error_template, {
                'came_from': came_from,
                }, context_instance=RequestContext(request))

    selected_idp = request.GET.get('idp', None)
    conf = config_loader()
    if selected_idp is None and conf.is_wayf_needed():
        return render_to_response(wayf_template, {
                'available_idps': conf.get_available_idps(),
                'came_from': came_from,
                }, context_instance=RequestContext(request))

    if selected_idp is not None:
        selected_idp = conf.single_sign_on_service(selected_idp)

    client = Saml2Client(conf)
    (session_id, result) = client.authenticate(
        location=selected_idp, relay_state=came_from,
        binding=BINDING_HTTP_REDIRECT,
        )
    assert len(result) == 2
    assert result[0] == 'Location'
    location = result[1]

    oq_cache = OutstandingQueriesCache(request.session)
    oq_cache.set(session_id, came_from)

    return HttpResponseRedirect(location)


DEFAULT_ATTRIBUTE_MAPPING = get_custom_setting('SAML_ATTRIBUTE_MAPPING',
                                               {'uid': 'username'})
DEFAULT_CREATE_UNKNOWN_USER = get_custom_setting('SAML_CREATE_UNKNOWN_USER',
                                                 True)


@csrf_exempt
def assertion_consumer_service(request, config_loader=config_settings_loader,
                               attribute_mapping=DEFAULT_ATTRIBUTE_MAPPING,
                               create_unknown_user=DEFAULT_CREATE_UNKNOWN_USER):
    """SAML Authorization Response endpoint

    The IdP will send its response to this view, which
    will process it with pysaml2 help and log the user
    in using the custom Authorization backend
    djangosaml2.backends.Saml2Backend that should be
    enabled in the settings.py
    """
    conf = config_loader()
    post = {'SAMLResponse': request.POST['SAMLResponse']}
    client = Saml2Client(conf, identity_cache=IdentityCache(request.session))

    oq_cache = OutstandingQueriesCache(request.session)
    outstanding_queries = oq_cache.outstanding_queries()

    # process the authentication response
    response = client.response(post, conf['entityid'], outstanding_queries)
    if response is None:
        return HttpResponse("SAML response has errors. Please check the logs")

    session_id = response.session_id()
    oq_cache.delete(session_id)

    # authenticate the remote user
    session_info = response.session_info()

    if 'djangosaml2.backends.Saml2Backend' not in settings.AUTHENTICATION_BACKENDS:
        settings.AUTHENTICATION_BACKENDS = (('djangosaml2.backends.Saml2Backend', )
                                            + settings.AUTHENTICATION_BACKENDS)

    if callable(attribute_mapping):
        attribute_mapping = attribute_mapping()
    if callable(create_unknown_user):
        create_unknown_user = create_unknown_user()

    user = auth.authenticate(session_info=session_info,
                             attribute_mapping=attribute_mapping,
                             create_unknown_user=create_unknown_user)
    if user is None:
        return HttpResponse("user not valid")

    auth.login(request, user)
    _set_subject_id(request.session, session_info['name_id'])

    post_authenticated.send_robust(sender=user, session_info=session_info)

    # redirect the user to the view where he came from
    relay_state = request.POST.get('RelayState', '/')
    return HttpResponseRedirect(relay_state)


@login_required
def echo_attributes(request,
                    config_loader=config_settings_loader,
                    template='djangosaml2/echo_attributes.html'):
    """Example view that echo the SAML attributes of an user"""
    state = StateCache(request.session)
    client = Saml2Client(config_loader(), state_cache=state,
                         identity_cache=IdentityCache(request.session))
    subject_id = _get_subject_id(request.session)
    identity = client.users.get_identity(subject_id,
                                         check_not_on_or_after=False)
    return render_to_response(template, {'attributes': identity[0]},
                              context_instance=RequestContext(request))


@login_required
def logout(request, config_loader=config_settings_loader):
    """SAML Logout Request initiator

    This view initiates the SAML2 Logout request
    using the pysaml2 library to create the LogoutRequest.
    """
    state = StateCache(request.session)
    client = Saml2Client(config_loader(), state_cache=state,
                         identity_cache=IdentityCache(request.session))
    subject_id = _get_subject_id(request.session)
    session_id, code, head, body = client.global_logout(subject_id)
    headers = dict(head)
    state.sync()
    return HttpResponseRedirect(headers['Location'])


def logout_service(request, config_loader=config_settings_loader,
                   next_page=None):
    """SAML Logout Response endpoint

    The IdP will send the logout response to this view,
    which will process it with pysaml2 help and log the user
    out.
    Note that the IdP can request a logout even when
    we didn't initiate the process as a single logout
    request started by another SP.
    """
    conf = config_loader()
    state = StateCache(request.session)
    client = Saml2Client(conf, state_cache=state,
                         identity_cache=IdentityCache(request.session))

    if 'SAMLResponse' in request.GET:  # we started the logout
        response = client.logout_response(request.GET['SAMLResponse'],
                                          binding=BINDING_HTTP_REDIRECT)
        state.sync()
        if response and response[1] == '200 Ok':
            return django_logout(request, next_page=next_page)
        else:
            return HttpResponse('Error during logout')

    elif 'SAMLRequest' in request.GET:  # logout started by the IdP
        subject_id = _get_subject_id(request.session)
        response, success = client.logout_request(request.GET, subject_id)
        state.sync()
        if success:
            auth.logout(request)
            assert response[0][0] == 'Location'
            url = response[0][1]
            return HttpResponseRedirect(url)
        elif response is not None:
            assert response[0][0] == 'Location'
            url = response[0][1]
            return HttpResponseRedirect(url)
        else:
            return HttpResponse('Error during logout')
    else:
        raise Http404('No SAMLResponse or SAMLRequest parameter found')


def metadata(request, config_loader=config_settings_loader):
    """Returns an XML with the SAML 2.0 metadata for this
    SP as configured in the settings.py file.
    """
    conf = config_loader()
    valid_for = conf.get('valid_for', 24)
    metadata = entity_descriptor(conf, valid_for)
    return HttpResponse(content=str(metadata),
                        content_type="text/xml; charset=utf8")


def register_namespace_prefixes():
    from saml2 import md, saml, samlp
    import xmlenc
    import xmldsig
    prefixes = (('saml', saml.NAMESPACE),
                ('samlp', samlp.NAMESPACE),
                ('md', md.NAMESPACE),
                ('ds', xmldsig.NAMESPACE),
                ('xenc', xmlenc.NAMESPACE))
    if hasattr(ElementTree, 'register_namespace'):
        for prefix, namespace in prefixes:
            ElementTree.register_namespace(prefix, namespace)
    else:
        for prefix, namespace in prefixes:
            ElementTree._namespace_map[namespace] = prefix

register_namespace_prefixes()



# Copyright (C) 2011 Yaco Sistemas (http://www.yaco.es)
# Copyright (C) 2010 Lorenzo Gil Sanchez <lorenzo.gil.sanchez@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from saml2.cache import Cache


class DjangoSessionCacheAdapter(dict):
    """A cache of things that are stored in the Django Session"""

    key_prefix = '_saml2'

    def __init__(self, django_session, key_suffix):
        self.session = django_session
        self.key = self.key_prefix + key_suffix

        super(DjangoSessionCacheAdapter, self).__init__(self._get_objects())

    def _get_objects(self):
        return self.session.get(self.key, {})

    def _set_objects(self, objects):
        self.session[self.key] = objects

    def sync(self):
        objs = {}
        objs.update(self)
        self._set_objects(objs)


class OutstandingQueriesCache(object):
    """Handles the queries that have been sent to the IdP and have not
    been replied yet.
    """

    def __init__(self, django_session):
        self._db = DjangoSessionCacheAdapter(django_session,
                                             '_outstanding_queries')

    def outstanding_queries(self):
        return self._db._get_objects()

    def set(self, saml2_session_id, came_from):
        self._db[saml2_session_id] = came_from
        self._db.sync()

    def delete(self, saml2_session_id):
        if saml2_session_id in self._db:
            del self._db[saml2_session_id]
            self._db.sync()


class IdentityCache(Cache):
    """Handles information about the users that have been succesfully
    logged in.

    This information is useful because when the user logs out we must
    know where does he come from in order to notify such IdP/AA.

    The current implementation stores this information in the Django session.
    """

    def __init__(self, django_session):
        self._db = DjangoSessionCacheAdapter(django_session, '_identities')
        self._sync = True

    def delete(self, subject_id):
        super(IdentityCache, self).delete(subject_id)
        # saml2.Cache doesn't do a sync after a delete
        # I'll send a patch to fix this in that side, after which this
        # could be removed
        self._db.sync()


class StateCache(DjangoSessionCacheAdapter):
    """Store state information that is needed to associate a logout
    request with its response.
    """

    def __init__(self, django_session):
        super(StateCache, self).__init__(django_session, '_state')



{
  'aRecord': 'urn:mace:dir:attribute-def:aRecord',
  'aliasedEntryName': 'urn:mace:dir:attribute-def:aliasedEntryName',
  'aliasedObjectName': 'urn:mace:dir:attribute-def:aliasedObjectName',
  'associatedDomain': 'urn:mace:dir:attribute-def:associatedDomain',
  'associatedName': 'urn:mace:dir:attribute-def:associatedName',
  'audio': 'urn:mace:dir:attribute-def:audio',
  'authorityRevocationList': 'urn:mace:dir:attribute-def:authorityRevocationList',
  'buildingName': 'urn:mace:dir:attribute-def:buildingName',
  'businessCategory': 'urn:mace:dir:attribute-def:businessCategory',
  'c': 'urn:mace:dir:attribute-def:c',
  'cACertificate': 'urn:mace:dir:attribute-def:cACertificate',
  'cNAMERecord': 'urn:mace:dir:attribute-def:cNAMERecord',
  'carLicense': 'urn:mace:dir:attribute-def:carLicense',
  'certificateRevocationList': 'urn:mace:dir:attribute-def:certificateRevocationList',
  'cn': 'urn:mace:dir:attribute-def:cn',
  'co': 'urn:mace:dir:attribute-def:co',
  'commonName': 'urn:mace:dir:attribute-def:commonName',
  'countryName': 'urn:mace:dir:attribute-def:countryName',
  'crossCertificatePair': 'urn:mace:dir:attribute-def:crossCertificatePair',
  'dITRedirect': 'urn:mace:dir:attribute-def:dITRedirect',
  'dSAQuality': 'urn:mace:dir:attribute-def:dSAQuality',
  'dc': 'urn:mace:dir:attribute-def:dc',
  'deltaRevocationList': 'urn:mace:dir:attribute-def:deltaRevocationList',
  'departmentNumber': 'urn:mace:dir:attribute-def:departmentNumber',
  'description': 'urn:mace:dir:attribute-def:description',
  'destinationIndicator': 'urn:mace:dir:attribute-def:destinationIndicator',
  'displayName': 'urn:mace:dir:attribute-def:displayName',
  'distinguishedName': 'urn:mace:dir:attribute-def:distinguishedName',
  'dmdName': 'urn:mace:dir:attribute-def:dmdName',
  'dnQualifier': 'urn:mace:dir:attribute-def:dnQualifier',
  'documentAuthor': 'urn:mace:dir:attribute-def:documentAuthor',
  'documentIdentifier': 'urn:mace:dir:attribute-def:documentIdentifier',
  'documentLocation': 'urn:mace:dir:attribute-def:documentLocation',
  'documentPublisher': 'urn:mace:dir:attribute-def:documentPublisher',
  'documentTitle': 'urn:mace:dir:attribute-def:documentTitle',
  'documentVersion': 'urn:mace:dir:attribute-def:documentVersion',
  'domainComponent': 'urn:mace:dir:attribute-def:domainComponent',
  'drink': 'urn:mace:dir:attribute-def:drink',
  'eduOrgHomePageURI': 'urn:mace:dir:attribute-def:eduOrgHomePageURI',
  'eduOrgIdentityAuthNPolicyURI': 'urn:mace:dir:attribute-def:eduOrgIdentityAuthNPolicyURI',
  'eduOrgLegalName': 'urn:mace:dir:attribute-def:eduOrgLegalName',
  'eduOrgSuperiorURI': 'urn:mace:dir:attribute-def:eduOrgSuperiorURI',
  'eduOrgWhitePagesURI': 'urn:mace:dir:attribute-def:eduOrgWhitePagesURI',
  'eduPersonAffiliation': 'urn:mace:dir:attribute-def:eduPersonAffiliation',
  'eduPersonEntitlement': 'urn:mace:dir:attribute-def:eduPersonEntitlement',
  'eduPersonNickname': 'urn:mace:dir:attribute-def:eduPersonNickname',
  'eduPersonOrgDN': 'urn:mace:dir:attribute-def:eduPersonOrgDN',
  'eduPersonOrgUnitDN': 'urn:mace:dir:attribute-def:eduPersonOrgUnitDN',
  'eduPersonPrimaryAffiliation': 'urn:mace:dir:attribute-def:eduPersonPrimaryAffiliation',
  'eduPersonPrimaryOrgUnitDN': 'urn:mace:dir:attribute-def:eduPersonPrimaryOrgUnitDN',
  'eduPersonPrincipalName': 'urn:mace:dir:attribute-def:eduPersonPrincipalName',
  'eduPersonScopedAffiliation': 'urn:mace:dir:attribute-def:eduPersonScopedAffiliation',
  'eduPersonTargetedID': 'urn:mace:dir:attribute-def:eduPersonTargetedID',
  'email': 'urn:mace:dir:attribute-def:email',
  'emailAddress': 'urn:mace:dir:attribute-def:emailAddress',
  'employeeNumber': 'urn:mace:dir:attribute-def:employeeNumber',
  'employeeType': 'urn:mace:dir:attribute-def:employeeType',
  'enhancedSearchGuide': 'urn:mace:dir:attribute-def:enhancedSearchGuide',
  'facsimileTelephoneNumber': 'urn:mace:dir:attribute-def:facsimileTelephoneNumber',
  'favouriteDrink': 'urn:mace:dir:attribute-def:favouriteDrink',
  'fax': 'urn:mace:dir:attribute-def:fax',
  'federationFeideSchemaVersion': 'urn:mace:dir:attribute-def:federationFeideSchemaVersion',
  'friendlyCountryName': 'urn:mace:dir:attribute-def:friendlyCountryName',
  'generationQualifier': 'urn:mace:dir:attribute-def:generationQualifier',
  'givenName': 'urn:mace:dir:attribute-def:givenName',
  'gn': 'urn:mace:dir:attribute-def:gn',
  'homePhone': 'urn:mace:dir:attribute-def:homePhone',
  'homePostalAddress': 'urn:mace:dir:attribute-def:homePostalAddress',
  'homeTelephoneNumber': 'urn:mace:dir:attribute-def:homeTelephoneNumber',
  'host': 'urn:mace:dir:attribute-def:host',
  'houseIdentifier': 'urn:mace:dir:attribute-def:houseIdentifier',
  'info': 'urn:mace:dir:attribute-def:info',
  'initials': 'urn:mace:dir:attribute-def:initials',
  'internationaliSDNNumber': 'urn:mace:dir:attribute-def:internationaliSDNNumber',
  'janetMailbox': 'urn:mace:dir:attribute-def:janetMailbox',
  'jpegPhoto': 'urn:mace:dir:attribute-def:jpegPhoto',
  'knowledgeInformation': 'urn:mace:dir:attribute-def:knowledgeInformation',
  'l': 'urn:mace:dir:attribute-def:l',
  'labeledURI': 'urn:mace:dir:attribute-def:labeledURI',
  'localityName': 'urn:mace:dir:attribute-def:localityName',
  'mDRecord': 'urn:mace:dir:attribute-def:mDRecord',
  'mXRecord': 'urn:mace:dir:attribute-def:mXRecord',
  'mail': 'urn:mace:dir:attribute-def:mail',
  'mailPreferenceOption': 'urn:mace:dir:attribute-def:mailPreferenceOption',
  'manager': 'urn:mace:dir:attribute-def:manager',
  'member': 'urn:mace:dir:attribute-def:member',
  'mobile': 'urn:mace:dir:attribute-def:mobile',
  'mobileTelephoneNumber': 'urn:mace:dir:attribute-def:mobileTelephoneNumber',
  'nSRecord': 'urn:mace:dir:attribute-def:nSRecord',
  'name': 'urn:mace:dir:attribute-def:name',
  'norEduOrgAcronym': 'urn:mace:dir:attribute-def:norEduOrgAcronym',
  'norEduOrgNIN': 'urn:mace:dir:attribute-def:norEduOrgNIN',
  'norEduOrgSchemaVersion': 'urn:mace:dir:attribute-def:norEduOrgSchemaVersion',
  'norEduOrgUniqueIdentifier': 'urn:mace:dir:attribute-def:norEduOrgUniqueIdentifier',
  'norEduOrgUniqueNumber': 'urn:mace:dir:attribute-def:norEduOrgUniqueNumber',
  'norEduOrgUnitUniqueIdentifier': 'urn:mace:dir:attribute-def:norEduOrgUnitUniqueIdentifier',
  'norEduOrgUnitUniqueNumber': 'urn:mace:dir:attribute-def:norEduOrgUnitUniqueNumber',
  'norEduPersonBirthDate': 'urn:mace:dir:attribute-def:norEduPersonBirthDate',
  'norEduPersonLIN': 'urn:mace:dir:attribute-def:norEduPersonLIN',
  'norEduPersonNIN': 'urn:mace:dir:attribute-def:norEduPersonNIN',
  'o': 'urn:mace:dir:attribute-def:o',
  'objectClass': 'urn:mace:dir:attribute-def:objectClass',
  'organizationName': 'urn:mace:dir:attribute-def:organizationName',
  'organizationalStatus': 'urn:mace:dir:attribute-def:organizationalStatus',
  'organizationalUnitName': 'urn:mace:dir:attribute-def:organizationalUnitName',
  'otherMailbox': 'urn:mace:dir:attribute-def:otherMailbox',
  'ou': 'urn:mace:dir:attribute-def:ou',
  'owner': 'urn:mace:dir:attribute-def:owner',
  'pager': 'urn:mace:dir:attribute-def:pager',
  'pagerTelephoneNumber': 'urn:mace:dir:attribute-def:pagerTelephoneNumber',
  'personalSignature': 'urn:mace:dir:attribute-def:personalSignature',
  'personalTitle': 'urn:mace:dir:attribute-def:personalTitle',
  'photo': 'urn:mace:dir:attribute-def:photo',
  'physicalDeliveryOfficeName': 'urn:mace:dir:attribute-def:physicalDeliveryOfficeName',
  'pkcs9email': 'urn:mace:dir:attribute-def:pkcs9email',
  'postOfficeBox': 'urn:mace:dir:attribute-def:postOfficeBox',
  'postalAddress': 'urn:mace:dir:attribute-def:postalAddress',
  'postalCode': 'urn:mace:dir:attribute-def:postalCode',
  'preferredDeliveryMethod': 'urn:mace:dir:attribute-def:preferredDeliveryMethod',
  'preferredLanguage': 'urn:mace:dir:attribute-def:preferredLanguage',
  'presentationAddress': 'urn:mace:dir:attribute-def:presentationAddress',
  'protocolInformation': 'urn:mace:dir:attribute-def:protocolInformation',
  'pseudonym': 'urn:mace:dir:attribute-def:pseudonym',
  'registeredAddress': 'urn:mace:dir:attribute-def:registeredAddress',
  'rfc822Mailbox': 'urn:mace:dir:attribute-def:rfc822Mailbox',
  'roleOccupant': 'urn:mace:dir:attribute-def:roleOccupant',
  'roomNumber': 'urn:mace:dir:attribute-def:roomNumber',
  'sOARecord': 'urn:mace:dir:attribute-def:sOARecord',
  'searchGuide': 'urn:mace:dir:attribute-def:searchGuide',
  'secretary': 'urn:mace:dir:attribute-def:secretary',
  'seeAlso': 'urn:mace:dir:attribute-def:seeAlso',
  'serialNumber': 'urn:mace:dir:attribute-def:serialNumber',
  'singleLevelQuality': 'urn:mace:dir:attribute-def:singleLevelQuality',
  'sn': 'urn:mace:dir:attribute-def:sn',
  'st': 'urn:mace:dir:attribute-def:st',
  'stateOrProvinceName': 'urn:mace:dir:attribute-def:stateOrProvinceName',
  'street': 'urn:mace:dir:attribute-def:street',
  'streetAddress': 'urn:mace:dir:attribute-def:streetAddress',
  'subtreeMaximumQuality': 'urn:mace:dir:attribute-def:subtreeMaximumQuality',
  'subtreeMinimumQuality': 'urn:mace:dir:attribute-def:subtreeMinimumQuality',
  'supportedAlgorithms': 'urn:mace:dir:attribute-def:supportedAlgorithms',
  'supportedApplicationContext': 'urn:mace:dir:attribute-def:supportedApplicationContext',
  'surname': 'urn:mace:dir:attribute-def:surname',
  'telephoneNumber': 'urn:mace:dir:attribute-def:telephoneNumber',
  'teletexTerminalIdentifier': 'urn:mace:dir:attribute-def:teletexTerminalIdentifier',
  'telexNumber': 'urn:mace:dir:attribute-def:telexNumber',
  'textEncodedORAddress': 'urn:mace:dir:attribute-def:textEncodedORAddress',
  'title': 'urn:mace:dir:attribute-def:title',
  'uid': 'urn:mace:dir:attribute-def:uid',
  'uniqueIdentifier': 'urn:mace:dir:attribute-def:uniqueIdentifier',
  'uniqueMember': 'urn:mace:dir:attribute-def:uniqueMember',
  'userCertificate': 'urn:mace:dir:attribute-def:userCertificate',
  'userClass': 'urn:mace:dir:attribute-def:userClass',
  'userPKCS12': 'urn:mace:dir:attribute-def:userPKCS12',
  'userPassword': 'urn:mace:dir:attribute-def:userPassword',
  'userSMIMECertificate': 'urn:mace:dir:attribute-def:userSMIMECertificate',
  'userid': 'urn:mace:dir:attribute-def:userid',
  'x121Address': 'urn:mace:dir:attribute-def:x121Address',
  'x500UniqueIdentifier': 'urn:mace:dir:attribute-def:x500UniqueIdentifier',
}



# Copyright (C) 2011 Yaco Sistemas (http://www.yaco.es)
# Copyright (C) 2010 Lorenzo Gil Sanchez <lorenzo.gil.sanchez@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import base64
import re
import urlparse

from django.conf import settings
from django.contrib.auth import SESSION_KEY
from django.contrib.auth.models import User
from django.test import TestCase

from saml2.s_utils import decode_base64_and_inflate, deflate_and_base64_encode

from djangosaml2 import views
from djangosaml2.cache import OutstandingQueriesCache
from djangosaml2.tests import conf
from djangosaml2.tests.auth_response import auth_response
from djangosaml2.signals import post_authenticated


class SAML2Tests(TestCase):

    urls = 'djangosaml2.urls'

    def assertSAMLRequestsEquals(self, xml1, xml2):

        def remove_variable_attributes(xml_string):
            xml_string = re.sub(r' ID=".*?" ', ' ', xml_string)
            xml_string = re.sub(r' IssueInstant=".*?" ', ' ', xml_string)
            xml_string = re.sub(
                r'<saml:NameID>.*</saml:NameID>',
                '<saml:NameID></saml:NameID>',
                xml_string)
            return xml_string

        self.assertEquals(remove_variable_attributes(xml1),
                          remove_variable_attributes(xml2))

    def init_cookies(self):
        self.client.cookies[settings.SESSION_COOKIE_NAME] = 'testing'

    def add_outstanding_query(self, session_id, came_from):
        session = self.client.session
        oq_cache = OutstandingQueriesCache(session)
        oq_cache.set(session_id, came_from)
        session.save()
        self.client.cookies[settings.SESSION_COOKIE_NAME] = session.session_key

    def test_login_one_idp(self):
        # monkey patch SAML configuration
        settings.SAML_CONFIG = conf.create_conf(sp_host='sp.example.com',
                                                idp_hosts=['idp.example.com'])

        response = self.client.get('/login/')
        self.assertEquals(response.status_code, 302)
        location = response['Location']

        url = urlparse.urlparse(location)
        self.assertEquals(url.hostname, 'idp.example.com')
        self.assertEquals(url.path, '/simplesaml/saml2/idp/SSOService.php')

        params = urlparse.parse_qs(url.query)
        self.assert_('SAMLRequest' in params)
        self.assert_('RelayState' in params)

        saml_request = params['SAMLRequest'][0]
        expected_request = """<?xml version='1.0' encoding='UTF-8'?>
<samlp:AuthnRequest xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" AssertionConsumerServiceURL="http://sp.example.com/saml2/acs/" Destination="https://idp.example.com/simplesaml/saml2/idp/SSOService.php" ID="XXXXXXXXXXXXXXXXXXXXXX" IssueInstant="2010-01-01T00:00:00Z" ProtocolBinding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST" ProviderName="Test SP" Version="2.0"><saml:Issuer Format="urn:oasis:names:tc:SAML:2.0:nameid-format:entity">http://sp.example.com/saml2/metadata/</saml:Issuer><samlp:NameIDPolicy AllowCreate="true" Format="urn:oasis:names:tc:SAML:2.0:nameid-format:transient" /></samlp:AuthnRequest>"""
        xml = decode_base64_and_inflate(saml_request)
        self.assertSAMLRequestsEquals(expected_request, xml)

        # if we set a next arg in the login view, it is preserverd
        # in the RelayState argument
        next = '/another-view/'
        response = self.client.get('/login/', {'next': next})
        self.assertEquals(response.status_code, 302)
        location = response['Location']

        url = urlparse.urlparse(location)
        self.assertEquals(url.hostname, 'idp.example.com')
        self.assertEquals(url.path, '/simplesaml/saml2/idp/SSOService.php')

        params = urlparse.parse_qs(url.query)
        self.assert_('SAMLRequest' in params)
        self.assert_('RelayState' in params)
        self.assertEquals(params['RelayState'][0], next)

    def test_login_several_idps(self):
        settings.SAML_CONFIG = conf.create_conf(sp_host='sp.example.com',
                                                idp_hosts=['idp1.example.com',
                                                           'idp2.example.com',
                                                           'idp3.example.com'])

        response = self.client.get('/login/')
        # a WAYF page should be displayed
        self.assertContains(response, 'Where are you from?', status_code=200)
        for i in range(1, 4):
            link = '/login/?idp=https://idp%d.example.com/simplesaml/saml2/idp/metadata.php&next=/'
            self.assertContains(response, link % i)

        # click on the second idp
        response = self.client.get('/login/', {
                'idp': 'https://idp2.example.com/simplesaml/saml2/idp/metadata.php',
                'next': '/',
                })
        self.assertEquals(response.status_code, 302)
        location = response['Location']

        url = urlparse.urlparse(location)
        self.assertEquals(url.hostname, 'idp2.example.com')
        self.assertEquals(url.path, '/simplesaml/saml2/idp/SSOService.php')

        params = urlparse.parse_qs(url.query)
        self.assert_('SAMLRequest' in params)
        self.assert_('RelayState' in params)

        saml_request = params['SAMLRequest'][0]
        expected_request = """<?xml version='1.0' encoding='UTF-8'?>
<samlp:AuthnRequest xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" AssertionConsumerServiceURL="http://sp.example.com/saml2/acs/" Destination="https://idp2.example.com/simplesaml/saml2/idp/SSOService.php" ID="XXXXXXXXXXXXXXXXXXXXXX" IssueInstant="2010-01-01T00:00:00Z" ProtocolBinding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST" ProviderName="Test SP" Version="2.0"><saml:Issuer Format="urn:oasis:names:tc:SAML:2.0:nameid-format:entity">http://sp.example.com/saml2/metadata/</saml:Issuer><samlp:NameIDPolicy AllowCreate="true" Format="urn:oasis:names:tc:SAML:2.0:nameid-format:transient" /></samlp:AuthnRequest>"""
        xml = decode_base64_and_inflate(saml_request)
        self.assertSAMLRequestsEquals(expected_request, xml)

    def test_assertion_consumer_service(self):
        # there are no users in the database
        self.assertEquals(User.objects.count(), 0)

        settings.SAML_CONFIG = conf.create_conf(sp_host='sp.example.com',
                                                idp_hosts=['idp.example.com'])

        config = views.config_settings_loader()
        # session_id should start with a letter since it is a NCName
        session_id = "a0123456789abcdef0123456789abcdef"
        came_from = '/another-view/'
        saml_response = auth_response({'uid': 'student'}, session_id, config)
        self.init_cookies()
        self.add_outstanding_query(session_id, came_from)

        # this will create a user
        response = self.client.post('/acs/', {
                'SAMLResponse': base64.b64encode(str(saml_response)),
                'RelayState': came_from,
                })
        self.assertEquals(response.status_code, 302)
        location = response['Location']

        url = urlparse.urlparse(location)
        self.assertEquals(url.hostname, 'testserver')
        self.assertEquals(url.path, came_from)

        self.assertEquals(User.objects.count(), 1)
        user_id = self.client.session[SESSION_KEY]
        user = User.objects.get(id=user_id)
        self.assertEquals(user.username, 'student')

        # let's create another user and log in with that one
        new_user = User.objects.create(username='teacher', password='not-used')

        session_id = "a1111111111111111111111111111111"
        came_from = '/'
        saml_response = auth_response({'uid': 'teacher'}, session_id, config)
        self.add_outstanding_query(session_id, came_from)
        response = self.client.post('/acs/', {
                'SAMLResponse': base64.b64encode(str(saml_response)),
                'RelayState': came_from,
                })
        self.assertEquals(response.status_code, 302)
        self.assertEquals(new_user.id, self.client.session[SESSION_KEY])

    def do_login(self):
        """Auxiliary method used in several tests (mainly logout tests)"""
        config = views.config_settings_loader()
        session_id = "a0123456789abcdef0123456789abcdef"
        came_from = '/another-view/'
        saml_response = auth_response({'uid': 'student'}, session_id, config)
        self.init_cookies()
        self.add_outstanding_query(session_id, came_from)

        # this will create a user
        response = self.client.post('/acs/', {
                'SAMLResponse': base64.b64encode(str(saml_response)),
                'RelayState': came_from,
                })
        self.assertEquals(response.status_code, 302)

    def test_logout(self):
        settings.SAML_CONFIG = conf.create_conf(sp_host='sp.example.com',
                                                idp_hosts=['idp.example.com'])

        self.do_login()

        response = self.client.get('/logout/')
        self.assertEquals(response.status_code, 302)
        location = response['Location']

        url = urlparse.urlparse(location)
        self.assertEquals(url.hostname, 'idp.example.com')
        self.assertEquals(url.path,
                          '/simplesaml/saml2/idp/SingleLogoutService.php')

        params = urlparse.parse_qs(url.query)
        self.assert_('SAMLRequest' in params)

        saml_request = params['SAMLRequest'][0]
        expected_request = """<?xml version='1.0' encoding='UTF-8'?>
<samlp:LogoutRequest xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" Destination="https://idp.example.com/simplesaml/saml2/idp/SingleLogoutService.php" ID="XXXXXXXXXXXXXXXXXXXXXX" IssueInstant="2010-01-01T00:00:00Z" Version="2.0"><saml:Issuer Format="urn:oasis:names:tc:SAML:2.0:nameid-format:entity">http://sp.example.com/saml2/metadata/</saml:Issuer><saml:NameID>58bcc81ea14700f66aeb707a0eff1360</saml:NameID></samlp:LogoutRequest>"""
        xml = decode_base64_and_inflate(saml_request)
        self.assertSAMLRequestsEquals(expected_request, xml)

    def test_logout_service_local(self):
        settings.SAML_CONFIG = conf.create_conf(sp_host='sp.example.com',
                                                idp_hosts=['idp.example.com'])

        self.do_login()

        response = self.client.get('/logout/')
        self.assertEquals(response.status_code, 302)
        location = response['Location']

        url = urlparse.urlparse(location)
        self.assertEquals(url.hostname, 'idp.example.com')
        self.assertEquals(url.path,
                          '/simplesaml/saml2/idp/SingleLogoutService.php')

        params = urlparse.parse_qs(url.query)
        self.assert_('SAMLRequest' in params)

        saml_request = params['SAMLRequest'][0]
        expected_request = """<?xml version='1.0' encoding='UTF-8'?>
<samlp:LogoutRequest xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" Destination="https://idp.example.com/simplesaml/saml2/idp/SingleLogoutService.php" ID="XXXXXXXXXXXXXXXXXXXXXX" IssueInstant="2010-01-01T00:00:00Z" Version="2.0"><saml:Issuer Format="urn:oasis:names:tc:SAML:2.0:nameid-format:entity">http://sp.example.com/saml2/metadata/</saml:Issuer><saml:NameID>58bcc81ea14700f66aeb707a0eff1360</saml:NameID></samlp:LogoutRequest>"""

        xml = decode_base64_and_inflate(saml_request)
        self.assertSAMLRequestsEquals(expected_request, xml)

        # now simulate a logout response sent by the idp
        request_id = re.findall(r' ID="(.*?)" ', xml)[0]
        instant = datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%SZ')

        saml_response = """<?xml version='1.0' encoding='UTF-8'?>
<samlp:LogoutResponse xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" Destination="http://sp.example.com/saml2/ls/" ID="a140848e7ce2bce834d7264ecdde0151" InResponseTo="%s" IssueInstant="%s" Version="2.0"><saml:Issuer Format="urn:oasis:names:tc:SAML:2.0:nameid-format:entity">https://idp.example.com/simplesaml/saml2/idp/metadata.php</saml:Issuer><samlp:Status><samlp:StatusCode Value="urn:oasis:names:tc:SAML:2.0:status:Success" /></samlp:Status></samlp:LogoutResponse>""" % (
            request_id, instant)

        response = self.client.get('/ls/', {
                'SAMLResponse': deflate_and_base64_encode(saml_response),
                })
        self.assertContains(response, "Logged out", status_code=200)
        self.assertEquals(self.client.session.keys(), [])

    def test_logout_service_global(self):
        settings.SAML_CONFIG = conf.create_conf(sp_host='sp.example.com',
                                                idp_hosts=['idp.example.com'])

        self.do_login()

        # now simulate a global logout process initiated by another SP
        subject_id = views._get_subject_id(self.client.session)
        instant = datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%SZ')
        saml_request = '<samlp:LogoutRequest xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" ID="_9961abbaae6d06d251226cb25e38bf8f468036e57e" Version="2.0" IssueInstant="%s" Destination="http://sp.example.com/saml2/ls/"><saml:Issuer>https://idp.example.com/simplesaml/saml2/idp/metadata.php</saml:Issuer><saml:NameID SPNameQualifier="http://sp.example.com/saml2/metadata/" Format="urn:oasis:names:tc:SAML:2.0:nameid-format:transient">%s</saml:NameID><samlp:SessionIndex>_1837687b7bc9faad85839dbeb319627889f3021757</samlp:SessionIndex></samlp:LogoutRequest>' % (
            instant, subject_id)

        response = self.client.get('/ls/', {
                'SAMLRequest': deflate_and_base64_encode(saml_request),
                })
        self.assertEquals(response.status_code, 302)
        location = response['Location']

        url = urlparse.urlparse(location)
        self.assertEquals(url.hostname, 'idp.example.com')
        self.assertEquals(url.path,
                          '/simplesaml/saml2/idp/SingleLogoutService.php')

        params = urlparse.parse_qs(url.query)
        self.assert_('SAMLResponse' in params)

        saml_response = params['SAMLResponse'][0]
        expected_response = """<?xml version='1.0' encoding='UTF-8'?>
<samlp:LogoutResponse xmlns:saml="urn:oasis:names:tc:SAML:2.0:assertion" xmlns:samlp="urn:oasis:names:tc:SAML:2.0:protocol" Destination="https://idp.example.com/simplesaml/saml2/idp/SingleLogoutService.php" ID="a140848e7ce2bce834d7264ecdde0151" InResponseTo="_9961abbaae6d06d251226cb25e38bf8f468036e57e" IssueInstant="2010-09-05T09:10:12Z" Version="2.0"><saml:Issuer Format="urn:oasis:names:tc:SAML:2.0:nameid-format:entity">http://sp.example.com/saml2/metadata/</saml:Issuer><samlp:Status><samlp:StatusCode Value="urn:oasis:names:tc:SAML:2.0:status:Success" /></samlp:Status></samlp:LogoutResponse>"""
        xml = decode_base64_and_inflate(saml_response)
        self.assertSAMLRequestsEquals(expected_response, xml)

    def test_metadata(self):
        settings.SAML_CONFIG = conf.create_conf(sp_host='sp.example.com',
                                                idp_hosts=['idp.example.com'])

        valid_until = datetime.datetime.utcnow() + datetime.timedelta(hours=24)
        valid_until = valid_until.strftime("%Y-%m-%dT%H:%M:%SZ")
        expected_metadata = """<?xml version='1.0' encoding='UTF-8'?>
<md:EntityDescriptor xmlns:ds="http://www.w3.org/2000/09/xmldsig#" xmlns:md="urn:oasis:names:tc:SAML:2.0:metadata" entityID="http://sp.example.com/saml2/metadata/" validUntil="%s"><md:SPSSODescriptor AuthnRequestsSigned="false" WantAssertionsSigned="true" protocolSupportEnumeration="urn:oasis:names:tc:SAML:2.0:protocol"><md:KeyDescriptor><ds:KeyInfo><ds:X509Data><ds:X509Certificate>MIIDPjCCAiYCCQCkHjPQlll+mzANBgkqhkiG9w0BAQUFADBhMQswCQYDVQQGEwJF
UzEQMA4GA1UECBMHU2V2aWxsYTEbMBkGA1UEChMSWWFjbyBTaXN0ZW1hcyBTLkwu
MRAwDgYDVQQHEwdTZXZpbGxhMREwDwYDVQQDEwh0aWNvdGljbzAeFw0wOTEyMDQx
OTQzNTJaFw0xMDEyMDQxOTQzNTJaMGExCzAJBgNVBAYTAkVTMRAwDgYDVQQIEwdT
ZXZpbGxhMRswGQYDVQQKExJZYWNvIFNpc3RlbWFzIFMuTC4xEDAOBgNVBAcTB1Nl
dmlsbGExETAPBgNVBAMTCHRpY290aWNvMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8A
MIIBCgKCAQEA7rMOMOaIZ/YYD5hYS6Hpjpovcu4k8gaIY+om9zCxLV5F8BLEfkxo
Pk9IA3cRQNRxf7AXCFxEOH3nKy56AIi1gU7X6fCT30JBT8NQlYdgOVMLlR+tjy1b
YV07tDa9U8gzjTyKQHgVwH0436+rmSPnacGj3fMwfySTMhtmrJmax0bIa8EB+gY1
77DBtvf8dIZIXLlGMQFloZeUspvHOrgNoEA9xU4E9AanGnV9HeV37zv3mLDUOQLx
4tk9sMQmylCpij7WZmcOV07DyJ/cEmnvHSalBTcyIgkcwlhmjtSgfCy6o5zuWxYd
T9ia80SZbWzn8N6B0q+nq23+Oee9H0lvcwIDAQABMA0GCSqGSIb3DQEBBQUAA4IB
AQCQBhKOqucJZAqGHx4ybDXNzpPethszonLNVg5deISSpWagy55KlGCi5laio/xq
hHRx18eTzeCeLHQYvTQxw0IjZOezJ1X30DD9lEqPr6C+IrmZc6bn/pF76xsvdaRS
gduNQPT1B25SV2HrEmbf8wafSlRARmBsyUHh860TqX7yFVjhYIAUF/El9rLca51j
ljCIqqvT+klPdjQoZwODWPFHgute2oNRmoIcMjSnoy1+mxOC2Q/j7kcD8/etulg2
XDxB3zD81gfdtT8VBFP+G4UrBa+5zFk6fT6U8a7ZqVsyH+rCXAdCyVlEC4Y5fZri
ID4zT0FcZASGuthM56rRJJSx
</ds:X509Certificate></ds:X509Data></ds:KeyInfo></md:KeyDescriptor><md:SingleLogoutService Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect" Location="http://sp.example.com/saml2/ls/" /><md:AssertionConsumerService Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-POST" Location="http://sp.example.com/saml2/acs/" index="1" /><md:AttributeConsumingService index="1"><md:ServiceName xml:lang="en">Test SP</md:ServiceName><md:RequestedAttribute FriendlyName="uid" Name="0.9.2342.19200300.100.1.1" NameFormat="urn:oasis:names:tc:SAML:2.0:attrname-format:uri" isRequired="true" /><md:RequestedAttribute FriendlyName="eduPersonAffiliation" Name="urn:oid:1.3.6.1.4.1.5923.1.1.1.1" NameFormat="urn:oasis:names:tc:SAML:2.0:attrname-format:uri" isRequired="false" /></md:AttributeConsumingService></md:SPSSODescriptor><md:Organization><md:OrganizationName xml:lang="es">Ejemplo S.A.</md:OrganizationName><md:OrganizationName xml:lang="en">Example Inc.</md:OrganizationName><md:OrganizationDisplayName xml:lang="es">Ejemplo</md:OrganizationDisplayName><md:OrganizationDisplayName xml:lang="en">Example</md:OrganizationDisplayName><md:OrganizationURL xml:lang="es">http://www.example.es</md:OrganizationURL><md:OrganizationURL xml:lang="en">http://www.example.com</md:OrganizationURL></md:Organization><md:ContactPerson contactType="technical"><md:Company>Example Inc.</md:Company><md:GivenName>Technical givenname</md:GivenName><md:SurName>Technical surname</md:SurName><md:EmailAddress>technical@sp.example.com</md:EmailAddress></md:ContactPerson><md:ContactPerson contactType="administrative"><md:Company>Example Inc.</md:Company><md:GivenName>Administrative givenname</md:GivenName><md:SurName>Administrative surname</md:SurName><md:EmailAddress>administrative@sp.example.ccom</md:EmailAddress></md:ContactPerson></md:EntityDescriptor>"""

        expected_metadata = expected_metadata % valid_until

        response = self.client.get('/metadata/')
        self.assertEquals(response['Content-type'], 'text/xml; charset=utf8')
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.content, expected_metadata)

    def test_post_authenticated_signal(self):

        def signal_handler(signal, user, session_info):
            self.assertEquals(isinstance(user, User), True)

        post_authenticated.connect(signal_handler, dispatch_uid='test_signal')

        self.do_login()

        post_authenticated.disconnect(dispatch_uid='test_signal')



# Copyright (C) 2011 Yaco Sistemas (http://www.yaco.es)
# Copyright (C) 2010 Lorenzo Gil Sanchez <lorenzo.gil.sanchez@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from saml2 import saml, class_name
from saml2.config import IDPConfig
from saml2.assertion import Assertion
from saml2.s_utils import response_factory, sid
from saml2.s_utils import success_status_factory
from saml2.sigver import signed_instance_factory, pre_signature_part
from saml2.sigver import security_context

BASEDIR = os.path.dirname(os.path.abspath(__file__))


def auth_response(identity, in_response_to, sp_conf):
    """Generates a fresh signed authentication response"""
    sp_entity_id = sp_conf['entityid']
    idp_entity_id = sp_conf['service']['sp']['idp'].keys()[0]
    acs = sp_conf.endpoint('sp', 'assertion_consumer_service')[0]
    attribute_converters = sp_conf.attribute_converters()
    issuer = saml.Issuer(text=idp_entity_id, format=saml.NAMEID_FORMAT_ENTITY)
    response = response_factory(issuer=issuer,
                                in_response_to=in_response_to,
                                destination=acs,
                                status=success_status_factory())
    idp_conf = IDPConfig()
    name_form = "urn:oasis:names:tc:SAML:2.0:attrname-format:uri"
    idp_conf.load({
            'entityid': idp_entity_id,
            'xmlsec_binary': sp_conf['xmlsec_binary'],
            'attribute_map_dir': sp_conf['attribute_map_dir'],
            'service': {
                'idp': {
                    'endpoints': tuple(),
                    'policy':  {
                        'default': {
                            "lifetime": {"minutes": 15},
                            "attribute_restrictions": None,
                            "name_form": name_form,
                            }
                        }
                    },
                },
            'key_file': os.path.join(BASEDIR, 'idpcert.key'),
            'cert_file': os.path.join(BASEDIR, 'idpcert.pem'),
            'metadata': '',
            })

    ast = Assertion(identity)
    policy = idp_conf.idp_policy()
    ast.apply_policy(sp_entity_id, policy, {})
    name_id = saml.NameID(format=saml.NAMEID_FORMAT_TRANSIENT,
                          text=sid())

    authn_class = saml.AUTHN_PASSWORD
    authn_authn = 'http://idp.example.com/login/'
    assertion = ast.construct(sp_entity_id, in_response_to, acs,
                              name_id, attribute_converters, policy,
                              issuer=issuer,
                              authn_class=authn_class,
                              authn_auth=authn_authn)

    sec = security_context(idp_conf)

    assertion.signature = pre_signature_part(assertion.id, sec.my_cert, 1)
    to_sign = [(class_name(assertion), assertion.id)]
    response.assertion = assertion
    return signed_instance_factory(response, sec, to_sign)



{
  'urn:mace:dir:attribute-def:aRecord': 'aRecord',
  'urn:mace:dir:attribute-def:aliasedEntryName': 'aliasedEntryName',
  'urn:mace:dir:attribute-def:aliasedObjectName': 'aliasedObjectName',
  'urn:mace:dir:attribute-def:associatedDomain': 'associatedDomain',
  'urn:mace:dir:attribute-def:associatedName': 'associatedName',
  'urn:mace:dir:attribute-def:audio': 'audio',
  'urn:mace:dir:attribute-def:authorityRevocationList': 'authorityRevocationList',
  'urn:mace:dir:attribute-def:buildingName': 'buildingName',
  'urn:mace:dir:attribute-def:businessCategory': 'businessCategory',
  'urn:mace:dir:attribute-def:c': 'c',
  'urn:mace:dir:attribute-def:cACertificate': 'cACertificate',
  'urn:mace:dir:attribute-def:cNAMERecord': 'cNAMERecord',
  'urn:mace:dir:attribute-def:carLicense': 'carLicense',
  'urn:mace:dir:attribute-def:certificateRevocationList': 'certificateRevocationList',
  'urn:mace:dir:attribute-def:cn': 'cn',
  'urn:mace:dir:attribute-def:co': 'co',
  'urn:mace:dir:attribute-def:commonName': 'commonName',
  'urn:mace:dir:attribute-def:countryName': 'countryName',
  'urn:mace:dir:attribute-def:crossCertificatePair': 'crossCertificatePair',
  'urn:mace:dir:attribute-def:dITRedirect': 'dITRedirect',
  'urn:mace:dir:attribute-def:dSAQuality': 'dSAQuality',
  'urn:mace:dir:attribute-def:dc': 'dc',
  'urn:mace:dir:attribute-def:deltaRevocationList': 'deltaRevocationList',
  'urn:mace:dir:attribute-def:departmentNumber': 'departmentNumber',
  'urn:mace:dir:attribute-def:description': 'description',
  'urn:mace:dir:attribute-def:destinationIndicator': 'destinationIndicator',
  'urn:mace:dir:attribute-def:displayName': 'displayName',
  'urn:mace:dir:attribute-def:distinguishedName': 'distinguishedName',
  'urn:mace:dir:attribute-def:dmdName': 'dmdName',
  'urn:mace:dir:attribute-def:dnQualifier': 'dnQualifier',
  'urn:mace:dir:attribute-def:documentAuthor': 'documentAuthor',
  'urn:mace:dir:attribute-def:documentIdentifier': 'documentIdentifier',
  'urn:mace:dir:attribute-def:documentLocation': 'documentLocation',
  'urn:mace:dir:attribute-def:documentPublisher': 'documentPublisher',
  'urn:mace:dir:attribute-def:documentTitle': 'documentTitle',
  'urn:mace:dir:attribute-def:documentVersion': 'documentVersion',
  'urn:mace:dir:attribute-def:domainComponent': 'domainComponent',
  'urn:mace:dir:attribute-def:drink': 'drink',
  'urn:mace:dir:attribute-def:eduOrgHomePageURI': 'eduOrgHomePageURI',
  'urn:mace:dir:attribute-def:eduOrgIdentityAuthNPolicyURI': 'eduOrgIdentityAuthNPolicyURI',
  'urn:mace:dir:attribute-def:eduOrgLegalName': 'eduOrgLegalName',
  'urn:mace:dir:attribute-def:eduOrgSuperiorURI': 'eduOrgSuperiorURI',
  'urn:mace:dir:attribute-def:eduOrgWhitePagesURI': 'eduOrgWhitePagesURI',
  'urn:mace:dir:attribute-def:eduPersonAffiliation': 'eduPersonAffiliation',
  'urn:mace:dir:attribute-def:eduPersonEntitlement': 'eduPersonEntitlement',
  'urn:mace:dir:attribute-def:eduPersonNickname': 'eduPersonNickname',
  'urn:mace:dir:attribute-def:eduPersonOrgDN': 'eduPersonOrgDN',
  'urn:mace:dir:attribute-def:eduPersonOrgUnitDN': 'eduPersonOrgUnitDN',
  'urn:mace:dir:attribute-def:eduPersonPrimaryAffiliation': 'eduPersonPrimaryAffiliation',
  'urn:mace:dir:attribute-def:eduPersonPrimaryOrgUnitDN': 'eduPersonPrimaryOrgUnitDN',
  'urn:mace:dir:attribute-def:eduPersonPrincipalName': 'eduPersonPrincipalName',
  'urn:mace:dir:attribute-def:eduPersonScopedAffiliation': 'eduPersonScopedAffiliation',
  'urn:mace:dir:attribute-def:eduPersonTargetedID': 'eduPersonTargetedID',
  'urn:mace:dir:attribute-def:email': 'email',
  'urn:mace:dir:attribute-def:emailAddress': 'emailAddress',
  'urn:mace:dir:attribute-def:employeeNumber': 'employeeNumber',
  'urn:mace:dir:attribute-def:employeeType': 'employeeType',
  'urn:mace:dir:attribute-def:enhancedSearchGuide': 'enhancedSearchGuide',
  'urn:mace:dir:attribute-def:facsimileTelephoneNumber': 'facsimileTelephoneNumber',
  'urn:mace:dir:attribute-def:favouriteDrink': 'favouriteDrink',
  'urn:mace:dir:attribute-def:fax': 'fax',
  'urn:mace:dir:attribute-def:federationFeideSchemaVersion': 'federationFeideSchemaVersion',
  'urn:mace:dir:attribute-def:friendlyCountryName': 'friendlyCountryName',
  'urn:mace:dir:attribute-def:generationQualifier': 'generationQualifier',
  'urn:mace:dir:attribute-def:givenName': 'givenName',
  'urn:mace:dir:attribute-def:gn': 'gn',
  'urn:mace:dir:attribute-def:homePhone': 'homePhone',
  'urn:mace:dir:attribute-def:homePostalAddress': 'homePostalAddress',
  'urn:mace:dir:attribute-def:homeTelephoneNumber': 'homeTelephoneNumber',
  'urn:mace:dir:attribute-def:host': 'host',
  'urn:mace:dir:attribute-def:houseIdentifier': 'houseIdentifier',
  'urn:mace:dir:attribute-def:info': 'info',
  'urn:mace:dir:attribute-def:initials': 'initials',
  'urn:mace:dir:attribute-def:internationaliSDNNumber': 'internationaliSDNNumber',
  'urn:mace:dir:attribute-def:janetMailbox': 'janetMailbox',
  'urn:mace:dir:attribute-def:jpegPhoto': 'jpegPhoto',
  'urn:mace:dir:attribute-def:knowledgeInformation': 'knowledgeInformation',
  'urn:mace:dir:attribute-def:l': 'l',
  'urn:mace:dir:attribute-def:labeledURI': 'labeledURI',
  'urn:mace:dir:attribute-def:localityName': 'localityName',
  'urn:mace:dir:attribute-def:mDRecord': 'mDRecord',
  'urn:mace:dir:attribute-def:mXRecord': 'mXRecord',
  'urn:mace:dir:attribute-def:mail': 'mail',
  'urn:mace:dir:attribute-def:mailPreferenceOption': 'mailPreferenceOption',
  'urn:mace:dir:attribute-def:manager': 'manager',
  'urn:mace:dir:attribute-def:member': 'member',
  'urn:mace:dir:attribute-def:mobile': 'mobile',
  'urn:mace:dir:attribute-def:mobileTelephoneNumber': 'mobileTelephoneNumber',
  'urn:mace:dir:attribute-def:nSRecord': 'nSRecord',
  'urn:mace:dir:attribute-def:name': 'name',
  'urn:mace:dir:attribute-def:norEduOrgAcronym': 'norEduOrgAcronym',
  'urn:mace:dir:attribute-def:norEduOrgNIN': 'norEduOrgNIN',
  'urn:mace:dir:attribute-def:norEduOrgSchemaVersion': 'norEduOrgSchemaVersion',
  'urn:mace:dir:attribute-def:norEduOrgUniqueIdentifier': 'norEduOrgUniqueIdentifier',
  'urn:mace:dir:attribute-def:norEduOrgUniqueNumber': 'norEduOrgUniqueNumber',
  'urn:mace:dir:attribute-def:norEduOrgUnitUniqueIdentifier': 'norEduOrgUnitUniqueIdentifier',
  'urn:mace:dir:attribute-def:norEduOrgUnitUniqueNumber': 'norEduOrgUnitUniqueNumber',
  'urn:mace:dir:attribute-def:norEduPersonBirthDate': 'norEduPersonBirthDate',
  'urn:mace:dir:attribute-def:norEduPersonLIN': 'norEduPersonLIN',
  'urn:mace:dir:attribute-def:norEduPersonNIN': 'norEduPersonNIN',
  'urn:mace:dir:attribute-def:o': 'o',
  'urn:mace:dir:attribute-def:objectClass': 'objectClass',
  'urn:mace:dir:attribute-def:organizationName': 'organizationName',
  'urn:mace:dir:attribute-def:organizationalStatus': 'organizationalStatus',
  'urn:mace:dir:attribute-def:organizationalUnitName': 'organizationalUnitName',
  'urn:mace:dir:attribute-def:otherMailbox': 'otherMailbox',
  'urn:mace:dir:attribute-def:ou': 'ou',
  'urn:mace:dir:attribute-def:owner': 'owner',
  'urn:mace:dir:attribute-def:pager': 'pager',
  'urn:mace:dir:attribute-def:pagerTelephoneNumber': 'pagerTelephoneNumber',
  'urn:mace:dir:attribute-def:personalSignature': 'personalSignature',
  'urn:mace:dir:attribute-def:personalTitle': 'personalTitle',
  'urn:mace:dir:attribute-def:photo': 'photo',
  'urn:mace:dir:attribute-def:physicalDeliveryOfficeName': 'physicalDeliveryOfficeName',
  'urn:mace:dir:attribute-def:pkcs9email': 'pkcs9email',
  'urn:mace:dir:attribute-def:postOfficeBox': 'postOfficeBox',
  'urn:mace:dir:attribute-def:postalAddress': 'postalAddress',
  'urn:mace:dir:attribute-def:postalCode': 'postalCode',
  'urn:mace:dir:attribute-def:preferredDeliveryMethod': 'preferredDeliveryMethod',
  'urn:mace:dir:attribute-def:preferredLanguage': 'preferredLanguage',
  'urn:mace:dir:attribute-def:presentationAddress': 'presentationAddress',
  'urn:mace:dir:attribute-def:protocolInformation': 'protocolInformation',
  'urn:mace:dir:attribute-def:pseudonym': 'pseudonym',
  'urn:mace:dir:attribute-def:registeredAddress': 'registeredAddress',
  'urn:mace:dir:attribute-def:rfc822Mailbox': 'rfc822Mailbox',
  'urn:mace:dir:attribute-def:roleOccupant': 'roleOccupant',
  'urn:mace:dir:attribute-def:roomNumber': 'roomNumber',
  'urn:mace:dir:attribute-def:sOARecord': 'sOARecord',
  'urn:mace:dir:attribute-def:searchGuide': 'searchGuide',
  'urn:mace:dir:attribute-def:secretary': 'secretary',
  'urn:mace:dir:attribute-def:seeAlso': 'seeAlso',
  'urn:mace:dir:attribute-def:serialNumber': 'serialNumber',
  'urn:mace:dir:attribute-def:singleLevelQuality': 'singleLevelQuality',
  'urn:mace:dir:attribute-def:sn': 'sn',
  'urn:mace:dir:attribute-def:st': 'st',
  'urn:mace:dir:attribute-def:stateOrProvinceName': 'stateOrProvinceName',
  'urn:mace:dir:attribute-def:street': 'street',
  'urn:mace:dir:attribute-def:streetAddress': 'streetAddress',
  'urn:mace:dir:attribute-def:subtreeMaximumQuality': 'subtreeMaximumQuality',
  'urn:mace:dir:attribute-def:subtreeMinimumQuality': 'subtreeMinimumQuality',
  'urn:mace:dir:attribute-def:supportedAlgorithms': 'supportedAlgorithms',
  'urn:mace:dir:attribute-def:supportedApplicationContext': 'supportedApplicationContext',
  'urn:mace:dir:attribute-def:surname': 'surname',
  'urn:mace:dir:attribute-def:telephoneNumber': 'telephoneNumber',
  'urn:mace:dir:attribute-def:teletexTerminalIdentifier': 'teletexTerminalIdentifier',
  'urn:mace:dir:attribute-def:telexNumber': 'telexNumber',
  'urn:mace:dir:attribute-def:textEncodedORAddress': 'textEncodedORAddress',
  'urn:mace:dir:attribute-def:title': 'title',
  'urn:mace:dir:attribute-def:uid': 'uid',
  'urn:mace:dir:attribute-def:uniqueIdentifier': 'uniqueIdentifier',
  'urn:mace:dir:attribute-def:uniqueMember': 'uniqueMember',
  'urn:mace:dir:attribute-def:userCertificate': 'userCertificate',
  'urn:mace:dir:attribute-def:userClass': 'userClass',
  'urn:mace:dir:attribute-def:userPKCS12': 'userPKCS12',
  'urn:mace:dir:attribute-def:userPassword': 'userPassword',
  'urn:mace:dir:attribute-def:userSMIMECertificate': 'userSMIMECertificate',
  'urn:mace:dir:attribute-def:userid': 'userid',
  'urn:mace:dir:attribute-def:x121Address': 'x121Address',
  'urn:mace:dir:attribute-def:x500UniqueIdentifier': 'x500UniqueIdentifier',
}



# Copyright (C) 2011 Yaco Sistemas <lgs@yaco.es>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


setup(
    name='djangosaml2',
    version='0.2.4',
    description='pysaml2 integration in Django',
    long_description='\n\n'.join([read('README'), read('CHANGES')]),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Security",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        ],
    keywords="pysaml2 django",
    author="Yaco Sistemas",
    author_email="lgs@yaco.es",
    url="https://bitbucket.org/lgs/djangosaml2",
    license='Apache 2.0',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'pysaml2',
        ],
    )



# Copyright (C) 2011 Yaco Sistemas (http://www.yaco.es)
# Copyright (C) 2010 Lorenzo Gil Sanchez <lorenzo.gil.sanchez@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os.path

import saml2


def create_conf(sp_host='sp.example.com', idp_hosts=['idp.example.com']):

    BASEDIR = os.path.dirname(os.path.abspath(__file__))
    config = {
        'xmlsec_binary': '/usr/bin/xmlsec1',
        'entityid': 'http://%s/saml2/metadata/' % sp_host,
        'attribute_map_dir': os.path.join(BASEDIR, 'attribute-maps'),

        'service': {
            'sp': {
                'name': 'Test SP',
                'endpoints': {
                    'assertion_consumer_service': [
                        ('http://%s/saml2/acs/' % sp_host,
                         saml2.BINDING_HTTP_POST),
                        ],
                    'single_logout_service': [
                        ('http://%s/saml2/ls/' % sp_host,
                         saml2.BINDING_HTTP_REDIRECT),
                        ],
                    },
                'required_attributes': ['uid'],
                'optional_attributes': ['eduPersonAffiliation'],
                'idp': {}  # this is filled later
                },
            },

        'metadata': {
            'local': [os.path.join(BASEDIR, 'remote_metadata.xml')],
            },

        'debug': 1,

        # certificates
        'key_file': os.path.join(BASEDIR, 'mycert.key'),
        'cert_file': os.path.join(BASEDIR, 'mycert.pem'),

        # These fields are only used when generating the metadata
        'contact_person': [
            {'given_name': 'Technical givenname',
             'sur_name': 'Technical surname',
             'company': 'Example Inc.',
             'email_address': 'technical@sp.example.com',
             'contact_type': 'technical'},
            {'given_name': 'Administrative givenname',
             'sur_name': 'Administrative surname',
             'company': 'Example Inc.',
             'email_address': 'administrative@sp.example.ccom',
             'contact_type': 'administrative'},
            ],
        'organization': {
            'name': [('Ejemplo S.A.', 'es'), ('Example Inc.', 'en')],
            'display_name': [('Ejemplo', 'es'), ('Example', 'en')],
            'url': [('http://www.example.es', 'es'),
                    ('http://www.example.com', 'en')],
            },
        'valid_for': 24,  # hours
        }

    for idp in idp_hosts:
        entity_id = 'https://%s/simplesaml/saml2/idp/metadata.php' % idp
        config['service']['sp']['idp'][entity_id] = {
            'single_sign_on_service': {
                saml2.BINDING_HTTP_REDIRECT: 'https://%s/simplesaml/saml2/idp/SSOService.php' % idp,
                },
            'single_logout_service': {
                saml2.BINDING_HTTP_REDIRECT: 'https://%s/simplesaml/saml2/idp/SingleLogoutService.php' % idp,
                },
            }

    return config
